#ifndef __OSACCS18_H__
#define __OSACCS18_H__

/*************************************************************************************************************************/
//------------------------------------------------------------------------------


//    FSR = FSR0
#device PIC18F252

#byte _fsr      = 0xFE9
#byte _indf     = 0xFEF
#byte _postinc  = 0xFEE
#byte _postdec  = 0xFED
#byte _preinc   = 0xFEC 

#byte _pcl      = 0xFF9
#byte _pclath   = 0xFFA
#byte _pclatu   = 0xFFB
#byte _status   = 0xFD8
#byte _tosl     = 0xFFD
#byte _tosh     = 0xFFE
#byte _tosu     = 0xFFF
#byte _bsr      = 0xFE0
#byte _wreg     = 0xFE8
#byte _intcon   = 0xFF2
#byte _stkptr   = 0xFFC
#byte _rcon     = 0xFD0

#define _FSR_TYPE            __osa_uint16
#define _PIC18_ERRATA_NOP()  #asm nop #endasm
#define _PIC18_POP()         #asm pop #endasm




#define __SMALL__

#if defined(__SMALL__)
    #define OST_CODE_POINTER    __osa_uint16    //     
    #define OS_CODE_POINTER_SIZE    2
#else
    #define OST_CODE_POINTER    __osa_uint32    //     
    #define OS_CODE_POINTER_SIZE    4
#endif


#define OST_TASK_POINTER   OS_TASKS_BANK OST_TCB *



#define OS_CLRWDT()   #asm  clrwdt  #endasm
#define OS_SLEEP()    #asm  sleep   #endasm







//______________________________________________________________________________
/******************************************************************************
 *       
 *
 *  _OS_JumpToTask() -     
 *  _OS_SaveRetAddr() -     (  )
 *
 *  _OS_GetReturnPoint() -       
 *  _OS_CallGetReturnPoint() -     
 *  _OS_ReturnNoSave() -       
 *
 *******************************************************************************/



#if defined(__SMALL__)
    #define _OS_SetPC()             \
        _pclath = _postdec;         \
        _pcl = _postdec;
    
    #define _OS_SavePC()            \
            _postdec = _tosh;       \
            _postdec = _tosl;

#else
    #define _OS_SetPC()             \
        _postdec = 0;               \
        _pclatu = _postdec;         \
        _pclath = _postdec;         \
        _pcl = _postdec;            \

    #define _OS_SavePC()            \
            _postdec = 0;           \
            _postdec = _tosu;       \
            _postdec = _tosh;       \
            _postdec = _tosl;


#endif


    #define _OS_JumpToTask()                                                    \
        {                                                                       \
            _fsr = (_FSR_TYPE)&_OS_CurTask->cTempFSR1;                          \
            #asm                                                                \
                bra     _OS_call_indirrect                                      \
            _OS_jump_indirrect:                                                 \
            #endasm                                                             \
            _wreg = _postdec;                                                   \
            _OS_SetPC();                                                        \
            /*  FSR   OS_CurTask->State    */                  \
            #asm                                                                \
            _OS_call_indirrect:                                                 \
                rcall   _OS_jump_indirrect                                      \
            #endasm                                                             \
        }

    #define _OS_SaveRetAddr()                                                   \
        _fsr = (_FSR_TYPE)&_OS_CurTask->cTempFSR1;                              \
        _postdec = _bsr;                                                        \
        _stkptr++;                                                              \
        _stkptr++;                                                              \
        _OS_SavePC();                                                           \
        _stkptr--;                                                              \
        _stkptr--;                                                              \

        
    #define _OS_if_Ready()  _OS_SET_FSR_CUR_TASK();                             \
                            if (_OS_CheckIndfReady())   


 
//------------------------------------------------------------------------------
extern void OS_SchedRetPoint (void);
extern void OS_SchedRetPoint2 (void);
extern void OS_ClearReady (void);
extern void OS_ClearReadySetCanContinue (void);


//------------------------------------------------------------------------------
#define _OS_GetReturnPoint()                    \
    {                                           \
        OS_SchedRetPoint();                     \
        _PIC18_ERRATA_NOP();                    \
    }
    
//------------------------------------------------------------------------------
#define _OS_CLEAR_READY()                       \
    {                                           \
        OS_ClearReady();                        \
        _PIC18_ERRATA_NOP();                    \
    }
//------------------------------------------------------------------------------
#define _OS_CLEAR_READY_SET_CANCONTINUE()       \
    {                                           \
        OS_ClearReadySetCanContinue();          \
        _PIC18_ERRATA_NOP();                    \
    }
//------------------------------------------------------------------------------
#define _OS_ReturnNoSave()                      \
    {                                           \
        OS_SchedRetPoint();                     \
        _PIC18_ERRATA_NOP();                    \
    }                                           \
//------------------------------------------------------------------------------
    
#define _OS_SET_FSR_CUR_TASK()      _fsr = (_FSR_TYPE)&_OS_CurTask->State
#define _OS_SET_FSR_CUR_TASK_W(w)   _fsr = ((_FSR_TYPE)&_OS_CurTask->State) + w
 



















//______________________________________________________________________________
/******************************************************************************************
 *
 *                                                
 *
 ******************************************************************************************/


 
 
/******************************************************************************************
 *
 *             P T I M E R '    
 *
 ******************************************************************************************/

#ifdef OS_ENABLE_PTIMERS


#if   OS_TIMER_SIZE == 1
    #define OS_INC_PTIMER()                                                             \
        {                                                                               \
            #asm                                                                        \
                movlw   3                                                               \
                addwf   _fsr, f                                                         \
                incf    _postdec, f     /*      FSR0L */    \
                movf    _postdec, w                                                     \
                movf    _postdec, w                                                     \
            #endasm                                                                     \
            if (_status & 0x01) _OS_SetIndfTimerTimeout;                                \
        }

        
#elif OS_TIMER_SIZE == 2
    #define OS_INC_PTIMER()                                                             \
        {                                                                               \
            _OS_PTimers.Flags.bTimeout = 0;                                             \
            #asm                                                                        \
                movlw   3                                                               \
                addwf   _fsr, f                                                         \
                movlw   0                                                               \
                incf    _postinc, f                                                     \
                addwfc  _postdec, f                                                     \
            #endasm                                                                     \
            if (_status & 0x01) _OS_PTimers.Flags.bTimeout = 1;                         \
            #asm                                                                        \
                movlw   3                                                               \
                subwf   _fsr, f                                                         \
            #endasm                                                                     \
            if (_OS_PTimers.Flags.bTimeout)                                             \
                _OS_SetIndfTimerTimeout;                                                \
        }


#elif OS_TIMER_SIZE == 4
    #define OS_INC_PTIMER()                                                             \
        {                                                                               \
            _OS_PTimers.Flags.bTimeout = 0;                                             \
            #asm                                                                        \
                movlw   3                                                               \
                addwf   _fsr, f                                                         \
                movlw   0                                                               \
                incf    _postinc, f                                                     \
                addwfc  _postinc, f                                                     \
                addwfc  _postinc, f                                                     \
                addwfc  _postdec, f                                                     \
            #endasm                                                                     \
            if (_status & 0x01) _OS_PTimers.Flags.bTimeout = 1;                         \
            #asm                                                                        \
                movlw   5                                                               \
                subwf   _fsr, f                                                         \
            #endasm                                                                     \
            if (_OS_PTimers.Flags.bTimeout)                                             \
                _OS_SetIndfTimerTimeout;                                                \
        }

#endif


#define _OS_PTIMERS_WORK()                              \
    {                                                   \
        _fsr = (_FSR_TYPE) &_OS_PTimers.Flags;          \
    REPEAT:                                             \
        if (_OS_CheckPostincTimerNextEnable)            \
        {                                               \
            /*  FSR  .  */     \
            #asm                                        \
                movf    _postinc, w                     \
                movff   _postdec, &_fsr + 1             \
                movwf   &_fsr + 0                       \
            #endasm                                     \
            if (!_OS_CheckIndfTimerRun) goto REPEAT;    \
            OS_INC_PTIMER();                            \
            goto REPEAT;                                \
        }                                               \
    }



#endif  // OS_ENABLE_PTIMER

/******************************************************************************************
 *
 *
 *
 ******************************************************************************************/





#endif
